SystemOrganization addCategory: #'Pier-Exporter-Code'!

!MAAccessor methodsFor: '*pier-exporter-code' stamp: 'NickAger 12/11/2011 22:52'!
asCodeFor: structure description: description value: value on: stream
	| structureClass |
	structureClass := structure class greaseString.
	stream << '	' << structureClass << ' description detect: [ :each | each label = ''' << description label << ''' ]) accessor write: ' << value << ' to: structure. '! !

GRObject subclass: #PRKernelCodeExporter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Exporter-Code'!

!PRKernelCodeExporter class methodsFor: 'helpers' stamp: 'NickAger 12/12/2011 16:05'!
selectorFromName: aName
	| selector |
	selector := aName select: [ :each | each isAlphaNumeric ].
	[ selector first isDigit ]
		whileTrue: [ selector := selector allButFirst ].
		
	selector at: 1 put: (selector at: 1) asLowercase.
		
	^ selector asSymbol! !

!PRKernelCodeExporter class methodsFor: 'helpers' stamp: 'NickAger 12/12/2011 16:07'!
selectorFromStructure: structure
	| aName |
	aName := structure absolutePath.
	aName := (aName = '/') ifTrue: [ structure name ] ifFalse: [ aName ].
	
	^  self selectorFromName: aName
! !

!PRKernelCodeExporter class methodsFor: 'helpers' stamp: 'NickAger 12/12/2011 16:19'!
writeCodeFor: structure selector: selector
	| methods method codeBuilder |
	methods := OrderedCollection new.
	
	method :=  String streamContents: [ :stream | 
		stream << selector << '
	| structure  |
	structure := ' << structure class greaseString << ' named: ''' << structure name << '''.
'.
		codeBuilder := PRCodeBuilderVisitor structure: structure methods: methods on: stream.
		structure description do: [ :aDescription |
			| accessor |
			accessor := aDescription accessor.
			(accessor canWrite: structure) ifTrue: [
				(accessor read: structure) notNil ifTrue: [
					codeBuilder visit: aDescription.
					stream << '
' ] ] ].
			stream << '	^ structure' ].
	
	methods add: method.		
	^ methods! !

!PRKernelCodeExporter methodsFor: 'exporting' stamp: 'NickAger 12/12/2011 15:25'!
compileMethods: methods intoComponent: component
	methods do: [ :aMethod |
		GRPlatform current 
			compile: aMethod
			into: component
			classified: #'exported code' ]



! !

!PRKernelCodeExporter methodsFor: 'exporting' stamp: 'NickAger 12/12/2011 16:26'!
exportKernel: aKernel
	| exportedCodeComponent rootComponent |
	exportedCodeComponent := PRKernelDistributionBase
		subclass: (self class selectorFromName: aKernel name) capitalized  
		instanceVariableNames: ''
		classVariableNames: 'rootPage'
		poolDictionaries: ''
		category: 'Pier-Exported-Code'.
	
	rootComponent := aKernel root.
		
	GRPlatform current 
		compile: (self rootPageTemplate: rootComponent)
		into: exportedCodeComponent
		classified: #'exported code'.
	
	rootComponent enumerator with all do: [ :structure |
		| selector methods |
		selector := self class selectorFromStructure: structure.
		Transcript show: selector greaseString; cr.
		methods := self class writeCodeFor: structure selector: selector.
		self compileMethods: methods intoComponent: exportedCodeComponent ]! !

!PRKernelCodeExporter methodsFor: 'exporting' stamp: 'NickAger 12/12/2011 16:06'!
rootPageTemplate: rootComponent
	^ String streamContents: [ :stream | 
		stream << 'rootPage
	^ rootPage ifNil: [
		rootPage := self ' << (self class selectorFromStructure: rootComponent) << ' ]']! !

MAVisitor subclass: #PRCodeBuilderVisitor
	instanceVariableNames: 'structure stream methods'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Exporter-Code'!

PRCodeBuilderVisitor class
	instanceVariableNames: 'aComponent'!

PRCodeBuilderVisitor class
	instanceVariableNames: 'aComponent'!

!PRCodeBuilderVisitor class methodsFor: 'instance creation' stamp: 'NickAger 12/12/2011 16:18'!
structure: structure methods: methods on: stream
	^ self basicNew
		initializeWithStructure: structure methods: methods stream: stream
		yourself! !

!PRCodeBuilderVisitor methodsFor: 'private' stamp: 'NickAger 12/12/2011 15:15'!
addCodeFor: aDescription value: value
	aDescription accessor asCodeFor: structure description: aDescription value: value on: stream! !

!PRCodeBuilderVisitor methodsFor: 'modifying' stamp: 'NickAger 12/12/2011 12:29'!
addMethod: method
	self methods add: method! !

!PRCodeBuilderVisitor methodsFor: 'initialization' stamp: 'NickAger 12/12/2011 16:18'!
initializeWithStructure: aStructure methods: methodsCollection stream: aStream
	self initialize.
	
	structure := aStructure.
	methods := methodsCollection.
	stream := aStream! !

!PRCodeBuilderVisitor methodsFor: 'initialization' stamp: 'NickAger 12/12/2011 12:27'!
initializeWithStructure: aStructure stream: aStream
	self initialize.
	
	structure := aStructure.
	stream := aStream! !

!PRCodeBuilderVisitor methodsFor: 'accessing' stamp: 'NickAger 12/12/2011 16:19'!
methods
	^ methods! !

!PRCodeBuilderVisitor methodsFor: 'visiting' stamp: 'NickAger 12/12/2011 15:16'!
visitDescription: aDescription
	| value |
	value := (aDescription accessor read: structure) asCode.
	self addCodeFor: aDescription value: value! !

!PRCodeBuilderVisitor methodsFor: 'visiting' stamp: 'NickAger 12/12/2011 15:15'!
visitDocumentDescription: aDescription
	"could have used:
		 value := (aDescription toString: (aDescription accessor read: structure)) asCode
	but I thought it was good to be explicit that we require wiki format output"
	| value |
	value := (PRWikiWriter write: (aDescription accessor read: structure)) asCode.
	self addCodeFor: aDescription value: value! !

!PRCodeBuilderVisitor methodsFor: 'visiting' stamp: 'NickAger 12/12/2011 15:19'!
visitFileDescription: aDescription
	| fileModel |
	fileModel := aDescription accessor read: structure.
	stream 
		<< '	structure file: ' << fileModel class greaseString << ' new
		filename: ' << fileModel filename asCode << ';
		mimetype: ' << fileModel mimetype asCode << ';
		filesize: ' << fileModel filesize asCode << ';
		contents: ' << (fileModel isText ifTrue: [ fileModel contents asString asCode ] ifFalse: [ fileModel contents greaseString ]) << '.'

! !

!PRCodeBuilderVisitor methodsFor: 'visiting' stamp: 'NickAger 12/12/2011 16:03'!
visitStructureDescription: aDescription
	| structureSelector subStructure |
	subStructure := aDescription accessor read: structure.
	structureSelector := PRKernelCodeExporter selectorFromStructure: subStructure.
	aDescription accessor asCodeFor: structure description: aDescription value: ' self ', structureSelector on: stream.
	self methods addAll: (PRKernelCodeExporter writeCodeFor: subStructure selector: structureSelector)! !

!MASelectorAccessor methodsFor: '*pier-exporter-code' stamp: 'NickAger 12/11/2011 22:52'!
asCodeFor: structure description: description value: value on: stream
	stream << '	structure ' << self writeSelector greaseString << ' ' << value << '.'! !

!Collection methodsFor: '*pier-exporter-code' stamp: 'NickAger 12/11/2011 20:49'!
codeOn: aStream
	aStream << '#('.
	self do: [ :each |
		each codeOn: aStream ].
	aStream << ')'! !

!PRDocumentAccessor methodsFor: '*pier-exporter-code' stamp: 'NickAger 12/11/2011 22:49'!
asCodeFor: structure description: description value: value on: stream
	stream << '	structure contents: ' << value << '.'! !
